%This routine computes the multigrid iteration matrix and its spectral radius
function brutaloptim2(n, nu)

disp(['n = ' num2str(n)]);
disp(['nu= ' num2str(nu)]);

ibase=200;
jbase=200;
idim=2*ibase
jdim=jbase;
rkstages=2
if (rkstages>2)
    kbase=50;
    kdim=kbase;
    if(rkstages>3)
        lbase=50;
        ldim=lbase;
    end
end

for i=1:idim
  cvec(i)=i/ibase;
end
for j=1:jdim
  avec(j)=j/jbase;
end

dx=2/n

A = (1+nu/dx)*eye(n) - nu/dx* diag(ones(n-1,1),-1);
A(1,n)=- nu/dx;
B=eye(n);
for i=1:n/2,
   R(i,2*i)=0.5;
   R(i,2*i-1)=0.5;
end
P=2*R';
RA=R*A;

n=n/2;
dx=2*dx;
Al = (1+nu/dx)*eye(n) - nu/dx* diag(ones(n-1,1),-1);
Al(1,n)=- nu/dx;
Bl=eye(n);
for i=1:n/2,
   Rl(i,2*i)=0.5;
   Rl(i,2*i-1)=0.5;
end
Pl=2*Rl';
RlAl=Rl*Al;

n=n/2;
dx=2*dx;
Al2 = (1+nu/dx)*eye(n) - nu/dx* diag(ones(n-1,1),-1);
Al2(1,n)=- nu/dx;
Bl2=eye(n);

n=n*4;
dx=dx/4;

for i=1:idim
  c=cvec(i);
  dt2=c*dx/nu;
  for j=1:jdim
    alpha=avec(j);
    if (rkstages==2)    
      %Second order polynomial for S
      S  =B - dt2*A + alpha*dt2^2*A^2;
      Sl =2*dt2*Bl -alpha* 4*dt2^2*Al;
      Sl2=4*dt2*Bl2-alpha*16*dt2^2*Al2;
      Mat=(B-P*(Sl+Pl*Sl2*Rl*(Bl-Al*Sl))*RA)*S;
      D=eig(Mat);
      optimat(i,j)=max(abs(D));
    elseif (rkstages==3)
      for k=1:kdim
        alpha2=k/kbase;
        %Third order polynomial for S
        S  =B - dt2*A + alpha2*dt2^2*A^2 - alpha*alpha2*dt2^3*A^3;
        Sl =2*dt2*Bl -alpha2* 4*dt2^2*Al +alpha*alpha2* 8*dt2^3*Al^2;
        Sl2=4*dt2*Bl2-alpha2*16*dt2^2*Al2+alpha*alpha2*64*dt2^3*Al2^2;
        Mat=(B-P*(Sl+Pl*Sl2*Rl*(Bl-Al*Sl))*RA)*S;
        D=eig(Mat);
        optimat(i,j,k)=max(abs(D));
      end
    elseif(rkstages==4)
      for k=1:kdim
        alpha2=k/kbase;
        for l=1:ldim
          alpha3=l/lbase;
          %Fourth order polynomial for S
          S  =B - dt2*A + alpha3*dt2^2*A^2 - alpha3*alpha2*dt2^3*A^3 + alpha*alpha2*alpha3*dt2^4*A^4;
          Sl =2*dt2*Bl -alpha3* 4*dt2^2*Al +alpha3*alpha2* 8*dt2^3*Al^2 +alpha*alpha2*alpha3*16*dt2^4*Al^3;
          Sl2=4*dt2*Bl2-alpha3*16*dt2^2*Al2+alpha3*alpha2*64*dt2^3*Al2^2+alpha*alpha2*alpha3*256*dt2^4*Al2^3;
          Mat=(B-P*(Sl+Pl*Sl2*Rl*(Bl-Al*Sl))*RA)*S;
          D=eig(Mat);
          optimat(i,j,k,l)=max(abs(D));
        end
      end
    end
  end
end

%Compute maxima
if (rkstages==2)
  %meshc(avec, cvec,log(optimat))
  [C, h]=contour(avec, cvec, log(optimat));
  set(h,'ShowText','on','TextStep',get(h,'LevelStep')*2)
  xlabel('\alpha')
  ylabel('c')
  grid on

  for i=1:idim
    maxvec((i-1)*jdim+1:i*jdim) = optimat(i,:);
  end
  [result resultindex]= min(maxvec)
  coptindex=ceil(resultindex/jdim)
  copt = coptindex/ibase
  alphaoptindex = resultindex-(coptindex-1)*jdim
  alphaopt = alphaoptindex/jbase
elseif (rkstages==3)
  for i=1:idim
    for j=1:jdim
      maxvec((i-1)*jdim*kdim + (j-1)*kdim + 1:(i-1)*jdim*kdim + j*kdim) = optimat(i,j,:);
    end
  end
  [result resultindex]= min(maxvec)
  coptindex=ceil(resultindex/(jdim*kdim))
  copt = coptindex/ibase
  alphaoptindex = ceil((resultindex-(coptindex-1)*jdim*kdim)/jdim)
  alphaopt = alphaoptindex/jbase
  alpha2optindex = resultindex-(coptindex-1)*jdim*kdim - (alphaoptindex-1)*jdim
  alpha2opt=alpha2optindex/kbase
elseif(rkstages==4)
  for i=1:idim
    for j=1:jdim
      for k=1:kdim
        maxvec((i-1)*jdim*kdim*ldim + (j-1)*kdim*ldim + (k-1)*ldim +1:(i-1)*jdim*kdim*ldim + (j-1)*kdim*ldim+k*ldim) = optimat(i,j,k,:);
      end
    end
  end
  [result resultindex]= min(maxvec)
  coptindex=ceil(resultindex/(jdim*kdim*ldim))
  copt = coptindex/ibase
  alphaoptindex = ceil((resultindex-(coptindex-1)*jdim*kdim*ldim)/(jdim*kdim))
  alphaopt = alphaoptindex/jbase
  alpha2optindex = ceil((resultindex-(coptindex-1)*jdim*kdim*ldim - (alphaoptindex-1)*jdim*kdim)/jdim)
  alpha2opt=alpha2optindex/kbase
  alpha3optindex=resultindex-(coptindex-1)*jdim*kdim*ldim - (alphaoptindex-1)*jdim*kdim-(alpha2optindex-1)*jdim
  alpha3opt=alpha3optindex/lbase
end