clear all
%Main file for solution of linear advection equation on [0,2]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%van-Leer/Lee/Powell, RK2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('VL2')
alpha=0.33333;
cflval=1.;
init
cflval=cflval*dx/nu;
error = norm(uexact-un);

%[V, D]=eig(-A);
%plot(diag(D),'*');
%evcoeff=V\(un-uexact);
%eigerrorfine=norm(evcoeff(1:n/2+1));
%eigerrorcoarse=norm(evcoeff(n/2+2:n));

%figure
%plot(un, 'bo')
%hold on
%plot(uexact, 'ro')

computeerror=-1;
for i=1:k,
  unew = multigrid(A, b, un, l, nu, nu1, alpha, cflval);
  un = unew;
  %  evcoeff=V\(unew-uexact);
%  eigerrorfine=[eigerrorfine; norm(evcoeff(1:n/2+1))];
%  eigerrorcoarse=[eigerrorcoarse; norm(evcoeff(n/2+2:n))];
  %this is to give out only every other data point
  computeerror=computeerror+1;
  if (computeerror==1)
     error = [error; norm(unew-uexact)];
     computeerror=-1;
  end
end

rho=nthroot(error(6)/error(1),10)
logrho=log10(rho)

%plot(unew, 'rx')
%legend('Sol. at t=0','Exact sol. at t=0.1','Discr. sol. at t=0.1')
%hold off

%figure
%hold on
%plot(log10(eigerrorfine), 'rx')
%plot(log10(eigerrorcoarse), 'bo')
%legend('Fine error components','Coarse error components')
%title('Van Leer et al (steady reference)')
%hold off

figure
hold on
plot(indexvec,log10(error),'r')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Second test case: !!!!!    VL3    !!!!!
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('VL3')
clear
init

error4 = norm(uexact-un)
 
computeerror=-1;
for i=1:k,
  unew = multigridVL3(A, b, un, l, nu, nu1);
  un = unew;
  computeerror=computeerror+1;
  if (computeerror==1)
     error4 = [error4; norm(unew-uexact)];
     computeerror=-1;
  end
end
 
rho=nthroot(error4(6)/error4(1),10)
logrho=log10(rho)

plot(indexvec,log10(error4),'rx-')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% !!!!!!    RK2 optimized, first method !!!!!!!!          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('RK2-opt')
clear
%CFL 3
alpha=0.3;
cflval=0.93;
%CFL 6
 alpha=0.315;
 cflval=0.96;
%CFL 24
% alpha=0.33;
% cflval=0.98;
init
cflval=cflval*dx/nu;

error2 = norm(uexact-un);

computeerror=-1;
for i=1:k,
  unew = multigrid(A, b, un, l, nu, nu1, alpha, cflval);
  un = unew;
  computeerror=computeerror+1;
  if (computeerror==1)
     error2 = [error2; norm(unew-uexact)];
     computeerror=-1;
  end
end

rho=nthroot(error2(6)/error2(1),10)
logrho=log10(rho)

plot(indexvec,log10(error2),'kv')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% !!!!!!    RK2 optimized, second method !!!!!!!!         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('RK2-rho-opt')
clear
% CFL=3
alpha=0.21;
cflval=0.615;
% CFL=6
 alpha=0.305;
 cflval=0.94;
% CFL=24
% alpha=0.295;
% cflval=1.46;
init
cflval=cflval*dx/nu;

error2b = norm(uexact-un);
 
computeerror=-1;
for i=1:k,
  unew = multigrid(A, b, un, l, nu, nu1, alpha, cflval);
  un = unew;
  computeerror=computeerror+1;
  if (computeerror==1)
     error2b = [error2b; norm(unew-uexact)];
     computeerror=-1;
  end
end
 
rho=nthroot(error2b(6)/error2b(1),10)
logrho=log10(rho)

plot(indexvec,log10(error2b),'.')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% !!!!!!    RK3 optimized, first method   !!!!!!!!  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('RK3-opt')
clear
%CFL 3
alpha(1)=0.135;
alpha(2)=0.375;
cflval=1.37;
%CFL 6
 alpha(1)=0.14;
 alpha(2)=0.385;
 cflval=1.445;
%CFL 24
% alpha(1)=0.145;
% alpha(2)=0.395;
% cflval=1.495;
init
cflval=cflval*dx/nu;

error3 = norm(uexact-un);

computeerror=-1;
for i=1:k,
  unew = multigrid(A, b, un, l, nu, nu1, alpha, cflval);
  un = unew;
  computeerror=computeerror+1;
  if (computeerror==1)
     error3 = [error3; norm(unew-uexact)];
     computeerror=-1;
  end
end

rho=nthroot(error3(6)/error3(1),10)
logrho=log10(rho)

plot(indexvec,log10(error3),'k^')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% !!!!!!    RK3 optimized, second method   !!!!!!!! %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('RK3-rho-opt')
clear
% CFL=3
alpha(1)=0.14;
alpha(2)=0.39;
cflval=1.42;
% CFL=6
 alpha(1)=0.14;
 alpha(2)=0.4;
 cflval=1.58;
% CFL=24
% alpha(1)=0.12;
% alpha(2)=0.38;
% cflval=2.14;
init
cflval=cflval*dx/nu;

error3b = norm(uexact-un);

computeerror=-1;
for i=1:k,
  unew = multigrid(A, b, un, l, nu, nu1, alpha, cflval);
  un = unew;
  computeerror=computeerror+1;
  if (computeerror==1)
     error3b = [error3b; norm(unew-uexact)];
     computeerror=-1;
  end
end

rho=nthroot(error3b(6)/error3b(1),10)
logrho=log10(rho)

plot(indexvec,log10(error3b),'o')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% !!!!!!    RK4 optimized, first method !!!!!!!!    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('RK4-opt')
clear
% CFL 3
alpha(1)=0.0733;
alpha(2)=0.1867;
alpha(3)=0.4;
cflval=1.96;
% CFL 6
 alpha(1)=0.08;
 alpha(2)=0.2;
 alpha(3)=0.4133;
 cflval=1.907;
% CFL 24
% alpha(1)=0.0867;
% alpha(2)=0.2133;
% alpha(3)=0.433;
% cflval=1.84;
init
cflval=cflval*dx/nu;

error5 = norm(uexact-un);

computeerror=-1;
for i=1:k,
  unew = multigrid(A, b, un, l, nu, nu1, alpha, cflval);
  un = unew;
  computeerror=computeerror+1;
  if (computeerror==1)
     error5 = [error5; norm(unew-uexact)];
     computeerror=-1;
  end
end

rho=nthroot(error5(6)/error5(1),10)
logrho=log10(rho)

plot(indexvec,log10(error5),'kp')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% !!!!!!    RK4 optimized, second method !!!!!!!!   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('RK4-rho-opt')
clear
% CFL=3
alpha(1)=0.02;
 alpha(2)=0.14;
 alpha(3)=0.38;
 cflval=1.42;
 % CFL=6
 alpha(1)=0.02;
  alpha(2)=0.14;
  alpha(3)=0.38;
  cflval=1.52;
% CFL=24
% alpha(1)=0.02;
%  alpha(2)=0.12;
%  alpha(3)=0.34;
%  cflval=2.16;
init
cflval=cflval*dx/nu;

error5b = norm(uexact-un);

computeerror=-1;
for i=1:k,
  unew = multigrid(A, b, un, l, nu, nu1, alpha, cflval);
  un = unew;
  computeerror=computeerror+1;
  if (computeerror==1)
     error5b = [error5b; norm(unew-uexact)];
     computeerror=-1;
  end
end

rho=nthroot(error5b(6)/error5b(1),10)
logrho=log10(rho)

plot(indexvec,log10(error5b),'*')

legend('RK2-VL', 'RK3-VL', 'RK2-opt', 'RK2-rho-opt', 'RK3-opt', 'RK3-rho-opt', 'RK4-opt', 'RK4-rho-opt')
xlabel('Iterations')
ylabel('Log of error')
hold off

%figure
%hold on
%plot(log10(eigerrorfine3), 'rx')
%plot(log10(eigerrorcoarse3), 'bo')
%legend('Fine error components','Coarse error components')
%title('Unsteady optimized method')
%hold off
